/**
  *  \file client/dialogs/gotoxydlg.cc
  *  \brief Dialog: Go To X/Y
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/dialogs/gotoxydlg.h"
#include "ui/window.h"
#include "ui/widgets.h"
#include "ui/msgbox.h"
#include "util/nls.h"
#include "ui/layout.h"
#include "game/coord.h"
#include "gfx/palette.h"
#include "ui/widgets/statictext.h"

namespace {
    class WGoToXYDialog : public UIWindow {
        ObjectHolder h;
        GPoint& result;
        UIInputLine* input;
     public:
        WGoToXYDialog(GPoint& result);
        void init();
        bool handleCommand(const UICommand& cmd, UIBaseWidget& sender);
    };
}

WGoToXYDialog::WGoToXYDialog(GPoint& result)
    : UIWindow(_("Go To X/Y"), UIWindow::BLUE_WINDOW, COLOR_GRAY),
      result(result),
      input(0)
{
    init();
}

void
WGoToXYDialog::init()
{
    // UIWindow
    //   UIMultilineStatic
    //   UIInputLine
    //   UIHBox
    //     UIButton "OK"
    //     UIButton "Cancel"
    //     UISpacer
    setLayout(UIVBoxLayout::instance5);

    add(h.add(new UIStaticText(_("Enter X,Y coordinates or sector number:"), UIStaticText::StaticNormal)));

    input = new UIInputLine(100, 8, 0, UIInputLine::ilf_Framed, 0);
    add(h.add(input));

    UIGroup& g = h.add(new UIGroup(UIHBoxLayout::instance5));
    add(g);
    g.add(h.add(new UIButton(1000, _("OK"), SDLK_RETURN)).addCommand());
    g.add(h.add(new UIButton(1001, _("Cancel"), SDLK_ESCAPE)).addCommand());
    g.add(h.add(new UISpacer()));

    pack();
}

bool
WGoToXYDialog::handleCommand(const UICommand& cmd, UIBaseWidget& sender)
{
    if (cmd.code == 1000) {
        // OK
        if (input->getText().size() == 0) {
            // ignore
        } else if (result.parseCoordinates(input->getText()) || result.parseSectorNumber(input->getText())) {
            // success
            stop(1);
        } else {
            // failure
            messageBox(_("Invalid input."), _("Go To X/Y"));
        }
        return true;
    } else if (cmd.code == 1001) {
        // Cancel
        stop(0);
        return true;
    } else {
        return UIWindow::handleCommand(cmd, sender);
    }
}

/** Perform "Go To X/Y" dialog. This dialog asks the user for a coordinate.
    \param out [out] Returns the coordinate if the user entered one
    \retval true when user entered a coordinate
    \retval false when user cancelled the dialog */
bool
doGoToXYDialog(GPoint& out)
{
    WGoToXYDialog dlg(out);
    ui_root->centerWidget(dlg);
    return ui_root->runModal(dlg);
}
