/**
  *  \file client/dlg-objsel.cc
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/dlg-objsel.h"
#include "ui/layout.h"
#include "gfx/palette.h"
#include "util/nls.h"
#include "ui/commands.h"
#include "game/obj.h"
#include "game/univ.h"
#include "ui/widgets.h"
#include "client/tiles/objheader.h"

WObjectSelectionDialog::WObjectSelectionDialog(const string_t title, GObjectType& type)
    : UIWindow(title, UIWindow::BLUE_WINDOW, COLOR_GRAY),
      type(type),
      content(h.add(new UIGroup(UIVBoxLayout::instance5)))
{
    buildDialog();
}

void
WObjectSelectionDialog::buildDialog()
{
    setLayout(UIVBoxLayout::instance5);

    UIGroup& buttons = h.add(new UIGroup(UIHBoxLayout::instance5));
    buttons.add(h.add(new UIButton(cm_Enter, _("ENTER - View"), SDLK_RETURN)).addStop());
    buttons.add(h.add(new UIButton(cm_Escape, _("ESC"),         SDLK_ESCAPE)).addStop());
    buttons.add(h.add(new UISpacer()));
    // buttons.add(h.add(new UIButton(cm_Help,  "H",               'H')).addCommand());

    add(h.add(new WObjectTypeHeaderTile(type)));
    add(content);
    add(buttons);
}

bool
WObjectSelectionDialog::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key) {
        switch (e.key.code) {
         case SDLK_PAGEUP:
         case SDLK_UP:
         case vk_WheelUp:
         case '-':
            type.setCurrent(type.getPreviousWrap(type.getCurrent(), false));
            return true;
         case SDLK_PAGEDOWN:
         case SDLK_DOWN:
         case vk_WheelDown:
         case '+':
            type.setCurrent(type.getNextWrap(type.getCurrent(), false));
            return true;
         case ss_Ctrl + SDLK_PAGEUP:
         case ss_Ctrl + SDLK_UP:
            type.setCurrent(type.getPreviousWrap(type.getCurrent(), true));
            return true;
         case ss_Ctrl + SDLK_PAGEDOWN:
         case ss_Ctrl + SDLK_DOWN:
            type.setCurrent(type.getNextWrap(type.getCurrent(), true));
            return true;
         case SDLK_HOME:
            type.setCurrent(type.getNextNoWrap(0, false));
            return true;
         case SDLK_END:
            type.setCurrent(type.getPreviousNoWrap(0, false));
            return true;
         case ss_Ctrl + SDLK_HOME:
            type.setCurrent(type.getNextNoWrap(0, true));
            return true;
         case ss_Ctrl + SDLK_END:
            type.setCurrent(type.getPreviousNoWrap(0, true));
            return true;
         case '.': {
            GObject& g = type.getObject(type.getCurrent());
            g.setIsMarked(!g.isMarked());
            type.getUniverse()->doScreenUpdates();
            return true;
         }
        }
    }
    return UIWindow::handleEvent(e, second_pass);
}

void
WObjectSelectionDialog::addWidget(UIBaseWidget& w)
{
    content.add(w);
}
