/**
  *  \file io/textfile.h
  *
  *  (c) 2001-2011 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_IO_TEXTFILE_H
#define PCC_V2_IO_TEXTFILE_H

#include <cstddef>
#include "cpluslib/string.h"
#include "util/recode.h"
#include "io/streams.h"

/** Text file access. Allows buffered access to a text file, supporting
    all possible newline conventions. */
class TextFile {
    enum Mode { Neutral, Reading, Writing };

    Stream&      stream;
    int          line_nr;
    std::size_t  buf_ptr;
    std::size_t  buf_fill;
    bool         eof;
    char         mode;
    char         buffer[1024];
    bool         utf8;
    CharacterSet cs;

    void refillBuffer();
    void flushBuffer();
    void setMode(Mode m);
 public:
    TextFile(Stream& s);
    ~TextFile();

    // Reading
    bool getLine(string_t& line);
    int  getLineNr() const;
    int  getByte();
    int  peekByte();
    std::size_t readBytes(void* data, std::size_t count);

    // Writing
    void writeLine(const string_t& line);
    void writeBytes(const void* data, std::size_t count);
    void sync();

    // Positioning
    Stream::fpos_t getPos();
    Stream::fpos_t getSize();
    bool seek(Stream::fpos_t pos);

    // Character set
    void setCharacterSet(CharacterSet cs);
    void setUtf8();
};

inline int
TextFile::getLineNr() const
{
    return line_nr;
}

#endif
